
#property  copyright ""
#property  link      ""

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  Green
#property indicator_color2  Red
#property indicator_color3  DarkSlateGray
#property indicator_color4  Gold
#property indicator_width1  2
#property indicator_width2  2

//
//
//
//
//

extern int     FastMaPeriod        = 5;
extern int     FastMaMode          = MODE_EMA;
extern int     FastMaPrice         = PRICE_MEDIAN;
extern int     SlowMaPeriod        = 34;
extern int     SlowMaMode          = MODE_EMA;
extern int     SlowMaPrice         = PRICE_MEDIAN;
extern int     SignalPeriod        = 5;
extern int     SignalMode          = MODE_SMMA;
extern int     SignalShift         = 0;

extern bool    alertsOn            = false;
extern bool    alertsOnSlope       = true;
extern bool    alertsOnSignalCross = true;
extern bool    alertsOnCurrent     = true;
extern bool    alertsMessage       = true;
extern bool    alertsSound         = false;
extern bool    alertsEmail         = false;
extern bool    arrowsVisible       = false;
extern bool    arrowsOnSlope       = true;
extern bool    arrowsOnSignalCross = true;
extern string  arrowsIdentifier    = "ao arrows1";
extern double  arrowsUpperGap      = 1.0;
extern double  arrowsLowerGap      = 1.0;
extern color   arrowsUpColor       = LimeGreen;
extern color   arrowsDnColor       = Red;
extern int     arrowsUpCode        = 241;
extern int     arrowsDnCode        = 242;

//
//
//
//
//

double ao[];
double aoUp[];
double aoDn[];
double sig[];
double slope[];
double trend[];

//
//
//
//
//

int init()
{
   IndicatorBuffers(6);   
   SetIndexBuffer(0,aoUp);SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,aoDn);SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,ao);
   SetIndexBuffer(3,sig); SetIndexShift(3,SignalShift);
   SetIndexBuffer(4,slope);
   SetIndexBuffer(5,trend);
   
   IndicatorShortName("AO");
return(0);
}

//
//
//
//
//

int deinit()
{
   deleteArrows();

return(0);
}

//+------------------------------------------------------------------+
//| Awesome Oscillator                                               |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //
       
   for(i=limit; i>=0; i--) ao[i] = iMA(NULL,0,FastMaPeriod,0,FastMaMode,FastMaPrice,i)-iMA(NULL,0,SlowMaPeriod,0,SlowMaMode,SlowMaPrice,i);
   for(i=limit; i>=0; i--)
   {
      sig[i] = iMAOnArray(ao,0,SignalPeriod,0,SignalMode,i);
      aoUp[i]  = EMPTY_VALUE;
      aoDn[i]  = EMPTY_VALUE;
      slope[i] = slope[i+1];
      trend[i] = trend[i+1];
         if (ao[i]>ao[i+1]) slope[i] = 1;
         if (ao[i]<ao[i+1]) slope[i] =-1;
         if (ao[i]>sig[i])  trend[i] = 1;
         if (ao[i]<sig[i])  trend[i] =-1;
         if (slope[i] == 1) aoUp[i]  = ao[i];
         if (slope[i] ==-1) aoDn[i]  = ao[i];
         manageArrow(i);
   }
   manageAlerts();
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      deleteArrow(Time[i]);
      string lookFor = arrowsIdentifier+":"+Time[i]; ObjectDelete(lookFor);
      if (arrowsOnSlope && slope[i] != slope[i+1])
      {
         if (slope[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
         if (slope[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
      } 
      if (arrowsOnSignalCross && trend[i] != trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
      }                 
   }         
}       

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
      static datetime time1 = 0;
      static string   mess1 = "";
         if (alertsOnSlope && slope[whichBar] != slope[whichBar+1])
         {
            if (slope[whichBar] ==  1) doAlert(time1,mess1,whichBar," awesome sloping up");
            if (slope[whichBar] == -1) doAlert(time1,mess1,whichBar," awesome sloping down");
         }            
      static datetime time2 = 0;
      static string   mess2 = "";
         if (alertsOnSignalCross && trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] ==  1) doAlert(time2,mess2,whichBar," awesome crossed signal line up");
            if (trend[whichBar] == -1) doAlert(time2,mess2,whichBar," awesome crossed signal line down");
         }            
   }
}   

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol(),TimeToStr(TimeLocal(),TIME_SECONDS)," Awesome trend changed to ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Awesome "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}