//+------------------------------------------------------------------+
//| Basket Profit 2 Pairs sq.mq4
//+------------------------------------------------------------------+

#property copyright "Squalou"
#property link      ""


/* 3-Pairs Basket Profit Indicator, by squalou:
 * draws a line representing the P/L of a 3-Pairs Basket, in Account Currency amount.
 * The light-blue vertical line is the Reference Line showing pointing the "Basket BE point",
 * typically put on the bar where you would "buy" this basket.
 * It can be moved manually to change the "Basket BE point", the indicator will automatically adjust the BE horizontal line (0.00 value)
 * accordingly at the next incoming tick;
 *
 * This was developped to help research on this Dreamliner's thread:
 * http://www.forexfactory.com/showthread.php?t=160912
 * and its EA dev thread from SteveHopwood:
 * http://www.forexfactory.com/showthread.php?t=278476
 * 
 */
 
/* - the Pair2 input will be updated automatically if the "Pair2" Label object on the chart is changed;
 * - the "Pair2" Label object is also used by the following Indicators/EAs to update their default "Pair2" input:
 *   - "Correlation Lines sq" indicator, 
 *   - "OverLayChart sq" indicator,
 *   - "Stochastic different pairs sq" indicator
 *   - "Stoch Diff 2 Pair EAsq" EA,
 * Changing the "Pair2" Label object text will automatically change the settings in those Indicators/EA if they are loaded on the chart.
 * Changing the "Pair2" input of any of these Indicators/EAs will have the same effect, and the "Pair2" Label object will be updated on the chart;
 *
 * - it is HIGHLY RECOMMENDED to load the indicators on the pair that has the highest tick refresh rate 
 *   to get the most reactive Indicators/EAs update -- especially when using an EA;
 * 
 * Note that the update of the Indicators/EAs will take effect only at the next incoming tick;
 * When markets are closed, it is therefore necessary to manually refresh the chart (right-click+Refresh),
 * or better use the "MT4_ticker.exe" program to generate artificial ticks;
 *
 * - You can load the "correlation stoch diff.tpl" template file on a chart to load all of the indicators above, but it will NOT load the EA;
 */


#property indicator_separate_window
//#property indicator_minimum -100
//#property indicator_maximum 100
//#property indicator_level1 -80
//#property indicator_level2 80
#property indicator_level3 0
#property indicator_buffers 1
#property indicator_color1 Red
#property indicator_color2 Orange
#property indicator_color3 MediumSlateBlue
#property indicator_color4 Green


extern string Pair1             = "";
extern double Lot1              = 1; // <0 means SELL, >0 means BUY
extern double Spread1           = 0;
extern string Pair2             = "";
extern double Lot2              = 1; // <0 means SELL, >0 means BUY
extern double Spread2           = 0;
extern string Pair3             = "";
extern double Lot3              = -1; // <0 means SELL, >0 means BUY
extern double Spread3           = 0;

extern int DaysBackReference    = 0; // where to put the "Reference Point 0"

double buffer1[];
string m = "";
bool stopped;

string ref_line = "Basket BE Reference";
int ref_bar;
datetime ref_time;
double ref1,ref2,ref3;
//double Point1,Point2;
double Value1,Value2,Value3;
//int pipMultTab[]={1,10,1,10,1,10,100}; // multiplier to convert pips to Points;

//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{
  if (Digits==3 || Digits==5)
   {
      Spread1=Spread1*10;
      Spread2=Spread2*10;
      Spread3=Spread3*10;
   }
  string m = StringSubstr(Symbol(), 6, StringLen(Symbol()) - 6);
  if (Pair1 == "") 
      Pair1 = Symbol();
  else 
      Pair1 = StringUpper(StringSubstr(Pair1,0,6))+m;
      Pair2 = StringUpper(StringSubstr(Pair2,0,6))+m;
      Pair3 = StringUpper(StringSubstr(Pair3,0,6))+m;

  set_Pair2(Pair2);

  IndicatorShortName("Basket "+AccountCurrency()+" ("+DoubleToStr(Lot1,2)+"*"+Pair1+DoubleToStr(Lot2,2)+"*"+Pair2
                     +DoubleToStr(Lot3,2)+"*"+Pair3+")");

  // plot BE Reference Line on DaysBackReference at 00:00
  ref_bar = iBarShift(NULL,0,iTime(NULL,PERIOD_D1,DaysBackReference));
  //ref_bar = DaysBackReference*PERIOD_D1/Period();
  ref_time = iTime(NULL,0,ref_bar);
  drawVLine(ref_line,ref_time,DodgerBlue,STYLE_SOLID);
  ref1 = iClose(Pair1,0,ref_bar);
  ref2 = iClose(Pair2,0,ref_bar);
  ref3 = iClose(Pair3,0,ref_bar);

  stopped=false;
  if (ref1==0) {
    Alert("!!! "+WindowExpertName()+" : no history data for "+Pair1+" !!!");
    stopped=true;
  }
  if (ref2==0) {
    Alert("!!! "+WindowExpertName()+" : no history data for "+Pair2+" !!!");
    stopped=true;
  }
  if (ref3==0) {
    Alert("!!! "+WindowExpertName()+" : no history data for "+Pair3+" !!!");
    stopped=true;
  }

  //---- indicators
  SetIndexBuffer(0, buffer1);
  SetIndexLabel(0,"Basket "+AccountCurrency()+" ("+DoubleToStr(Lot1,2)+"*"+Pair1+DoubleToStr(Lot2,2)+"*"+Pair2
                +DoubleToStr(Lot3,2)+"*"+Pair3+")");

  Value1 = PointValueFor1Lot(Pair1)/MarketInfo(Pair1,MODE_POINT);
  Value2 = PointValueFor1Lot(Pair2)/MarketInfo(Pair2,MODE_POINT);
  Value3 = PointValueFor1Lot(Pair3)/MarketInfo(Pair3,MODE_POINT);

  return (0);
}

//+------------------------------------------------------------------+
int deinit()
//+------------------------------------------------------------------+
{
  return (0);
}

//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
{
  if (stopped) return;

  int BarsBack = Bars;// DaysBackReference*PERIOD_D1/Period();
  
  int i, counted_bars = IndicatorCounted();
  if (counted_bars > 0) counted_bars--;
  int limit = Bars - counted_bars;
  limit = MathMin(limit,BarsBack);

  if (get_Pair2_from_chart(Pair2)) { // will update itself if "Pair2" label object was changed (manually or from another indicator/EA)
    deinit();
    init();
    limit=BarsBack;
  }

  if (ObjectFind(ref_line)>=0) {  
    double new_ref = ObjectGet(ref_line,OBJPROP_TIME1);
    if (new_ref!=ref_time) {
      ref_time = new_ref;
      Print("BE Reference Line moved to "+TimeToStr(ref_time,TIME_DATE|TIME_MINUTES));
      drawVLine(ref_line,ref_time,DodgerBlue,STYLE_SOLID);
      ref_bar = iBarShift(NULL,0,ref_time);
      ref1 = iClose(Pair1,0,ref_bar);
      ref2 = iClose(Pair2,0,ref_bar);
      ref3 = iClose(Pair3,0,ref_bar);
      limit=BarsBack;
    }
  }
  
  for(i=limit; i>=0; i--)
  {
    double curr1 = (iClose(Pair1,0,i)-ref1)*Value1*Lot1-(Spread1*MarketInfo(Pair1,MODE_TICKVALUE)*MathAbs(Lot1));
    double curr2 = (iClose(Pair2,0,i)-ref2)*Value2*Lot2-(Spread2*MarketInfo(Pair2,MODE_TICKVALUE)*MathAbs(Lot2));
    double curr3 = (iClose(Pair3,0,i)-ref3)*Value3*Lot3-(Spread3*MarketInfo(Pair3,MODE_TICKVALUE)*MathAbs(Lot3));

    buffer1[i] = curr1+curr2+curr3;
  }

  return (0);
}

//+------------------------------------------------------------------
void drawVLine(string objname, int time, color c, int style=STYLE_SOLID, int width=0)
//+------------------------------------------------------------------
{
  ObjectCreate(objname, OBJ_VLINE, 0, time, 0);
  ObjectSet(objname, OBJPROP_COLOR, c);
  ObjectSet(objname, OBJPROP_STYLE, style);
}

//+------------------------------------------------------------------+
string __m;
void set_Pair2(string &Pair2)
//+------------------------------------------------------------------+
{
  __m = StringSubstr(Symbol(), 6, StringLen(Symbol()) - 6);
  Pair2 = StringSubstr(Pair2,0,6) + __m;
  if (MarketInfo(Pair2,MODE_ASK)==0) {// Pair2 does not exist, get it from the chart "Pair2" object
    get_Pair2_from_chart(Pair2);
  } else {
    drawFixedLbl("Pair2", Pair2, 1, 8, 20, 14, "Arial", MediumVioletRed, false);
  }
}

//+------------------------------------------------------------------+
bool get_Pair2_from_chart(string &Pair2)
//+------------------------------------------------------------------+
{

  /* Import Pair2 input from the "Pair2" Label object if it exists on the chart
   * The "Pair2" Label is also used by "OverLayChart" indicator,
   * and the "Stochastic different pairs 1.6b.mq4" EA,
   * so that they can all be loaded on the same chart and run with the same second-pair
   */
  if (ObjectFind("Pair2")>=0) {
    string p2 = ObjectDescription("Pair2");
    if (p2!=Pair2) {
      Pair2 = StringSubstr(p2,0,6) + __m;
      if (MarketInfo(Pair2,MODE_ASK)==0) Pair2=Symbol();// Pair2 does not exist, default to chart's symbol
      drawFixedLbl("Pair2", Pair2, 1, 8, 20, 14, "Arial", MediumVioletRed, false);
      Print("Found \"Pair2\" object : "+Pair2);
      return(true);
    }
  } else { // "Pair2" object does not exist: create it
    Pair2 = StringSubstr(Pair2,0,6) + __m;
    if (MarketInfo(Pair2,MODE_ASK)==0) Pair2=Symbol();// Pair2 does not exist, default to chart's symbol
    drawFixedLbl("Pair2", Pair2, 1, 8, 20, 14, "Arial", MediumVioletRed, false);
  }
  return(false);
}

//--------------------------------------------------------------------------------------
void drawFixedLbl(string objname, string s, int Corner, int DX, int DY, int FSize, string Font, color c, bool bg)
//--------------------------------------------------------------------------------------
{
   if (ObjectFind(objname) < 0) ObjectCreate(objname, OBJ_LABEL, 0, 0, 0);
   
   ObjectSet(objname, OBJPROP_CORNER, Corner);
   ObjectSet(objname, OBJPROP_XDISTANCE, DX);
   ObjectSet(objname, OBJPROP_YDISTANCE, DY);
   ObjectSet(objname,OBJPROP_BACK, bg);      
   ObjectSetText(objname, s, FSize, Font, c);
} // drawFixedLbl

//+------------------------------------------------------------------+
double PointValueFor1Lot(string symbol)
//+------------------------------------------------------------------+
{
  // PtVal = TickValue * Point / TickSize;
  return ( MarketInfo(symbol,MODE_TICKVALUE)*MarketInfo(symbol,MODE_POINT)/MarketInfo(symbol,MODE_TICKSIZE) );
}
//+------------------------------------------------------------------+
string StringUpper(string str)
//+------------------------------------------------------------------+
// Converts any lowercase characters in a string to uppercase
// Usage:    string x=StringUpper("The Quick Brown Fox")  returns x = "THE QUICK BROWN FOX"
{
  string outstr = "";
  string lower  = "abcdefghijklmnopqrstuvwxyz";
  string upper  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  for(int i=0; i<StringLen(str); i++)  {
    int t1 = StringFind(lower,StringSubstr(str,i,1),0);
    if (t1 >=0)  
      outstr = outstr + StringSubstr(upper,t1,1);
    else
      outstr = outstr + StringSubstr(str,i,1);
  }
  return(outstr);
}

