//+------------------------------------------------------------------+
//|                                  Multi-purpose trade manager.mq4 |
//|                                  Copyright  2008, Steve Hopwood |
//|                              http://www.hopwood3.freeserve.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Steve Hopwood"
#property link      "http://www.hopwood3.freeserve.co.uk"
#include <WinUser32.mqh>
#include <stdlib.mqh>
#define  NL    "\n"


/*
This basket trade manager was extracted from Steve Hopwood's MPTM to work with Phantom Six EAs.
Somehow when the original MPTM is loaded in a chart, it would cause the Phantom EA to freeze. The GlobalVariables
are gone. - luiboy

*/



//+------------------------------------------------------------------+
//|              External Variables                                  |
//+------------------------------------------------------------------+

extern bool    ManageByMagicNumber=false;
extern int     MagicNumber=1;
extern bool    ManageByTradeComment=false;
extern string  TradeComment = "";
extern string  bl9="---------------------------------------------------------------------";
extern bool    ManageBasketTrades=true;
extern bool    IncludePendingsAtClosure=false;
extern string  bin5="Basket take profit settings";
extern bool    BasketClosureTP=false;
extern bool    BasketTPinDollars=false;
extern double  BasketDollarTP=100000;
extern bool    BasketTPasPercent=false;
extern double  BasketTpPercentage=1;
extern bool    AutoCalcBasketTPasPercent=false;
extern double  FourToSevenTradesPercent=1;
extern double  EightToTwelveTradesPercent=2;
extern double  ThirteenPlusTradesPercent=3;
extern string  bin6="Basket stop loss settings";
extern bool    BasketClosureSL=false;
extern bool    BasketSLinDollars=false;
extern double  BasketDollarSL=100000;
extern bool    BasketSLasPercent=false;
extern double  BasketSLPercentage=1;
extern string  bin13="Basket jumping stop settings";
extern bool    BasketJumpingStop=false;
extern double  BasketJumpingStopProfit=50;
extern bool    BasketAddBEP=false;
extern double  BasketBreakEvenProfit=50;
extern bool    DisableBasketJumpStopAfterBE=false;
extern string  bin7="Basket trailing stop settings";
extern bool    BasketTrailingStop=false;
extern double  BasketTsAtProfit=1;
extern double  BasketTrailPercent=75;
extern string  bin8="Immediate basket closure setting. Use with care";
extern bool    BasketCloseImmediately=false;
// Added by Robert for those who do not want the comments.
extern bool    ShowComments = true;
extern bool    ShowAlerts=true;


//+------------------------------------------------------------------+
//|              Internal Variables                                  |
//+------------------------------------------------------------------+


bool           CloseBasket=false;
double         BasketProfit;
string         ScreenMessage;
int            NoOfBasketTrades;//Holds the no of basket trades open. Set in CalculateBasketPL()
double         LockedProfit=-1;



//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   
    //Make sure the basket sl is a negativ number
   if (BasketDollarSL > 0) BasketDollarSL = -BasketDollarSL;
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----

   main(); 

//----
   return(0);
  }
//+------------------------------------------------------------------+\


void main()

{

 // Stop if there is nothing to do
   if (OrdersTotal()==0)
   {
      if (ShowComments) Comment("No trades to manage. I am bored witless.");
      return(0);
   }
   
   
   
 
   
   // Basket trading
   if (ManageBasketTrades) MonitorBasketTrades();
   
   
}//end void main()
   
   
   
void MonitorBasketTrades()
{
   
   // CloseBasket is set to true by any routine that finds the basket should be closed.
   // The function deals with cancelling Closebasket. This only happens when all trades
   // have been closed successfully.
   if (CloseBasket) 
   {
      CloseBasket = false;
      CloseBasketTrades();
      return;//Nothing else to do
   }

   // User has chosen to close basket trades
   if (BasketCloseImmediately)
   {
      bool BasketClosure=ConfirmBasketClosure();
      if (BasketClosure) 
      {
         Comment("Closing basket trades");
         CloseBasket = false;
         CloseBasketTrades();
         return;
      }   
   }//if (BasketCloseImmediately)

   // Calculate the upl for the basket.
   // Also calculates the number of trades in the basket and stores this in  NoOfBasketTrades
   CalculateBasketPL();
   
   // Check to see if the basket has reached a pre-determined profit point
   if (BasketClosureTP && BasketProfit>0)
   {
      BasketClosure=ShouldBasketCloseAtTP(); // Retruns true if the basket upl >= chosen closure point
      if (BasketClosure) 
      {
         Alert("Basket TP hit, so the trades were closed");
         CloseBasket = false;
         CloseBasketTrades();
         return;
      }   
   }//if (BasketClosureTP && BasketProfit>0)
   
   // Check to see if the basket has hit a pre-determined stop loss
   if (BasketClosureSL && BasketProfit<0)
   {
      BasketClosure=ShouldBasketCloseAtSL(); // Retruns true if the basket upl >= chosen closure point
      if (BasketClosure) 
      {
         Alert("Basket SL hit, so the trades were closed");
         CloseBasket = false;
         CloseBasketTrades();
         return;
      }   
   }//if (BasketClosureSL&& BasketProfit<0)
   
   // Check to see if the basket has reached an auto-calculated percentage of balance profit
   if (AutoCalcBasketTPasPercent && BasketProfit>0)
   {
      BasketClosure=ShouldBasketCloseAtAutocalcTP(); // Retruns true if the basket upl >= chosen closure point
      if (BasketClosure) 
      {
         Alert("Basket TP hit, so the trades were closed");
         CloseBasket = false;
         CloseBasketTrades();
         return;
      }   
   }//if (AutoCalcBasketTPasPercent && BasketProfit<0)
   
   // Trailing stop
   if (BasketTrailingStop)
   {
      BasketTrailingStopManipulation();
   }
   
   //Jumping stop loss
   if (BasketJumpingStop) BasketJumpingStopManipulation();
  
   
   
   // User feedback. Basket profit\loss is already displayed, so add details to ScreenMessage
   string SaveSM=ScreenMessage;
   ScreenMessage = StringConcatenate(ScreenMessage, "Managing basket trades by ");
   if (ManageByMagicNumber) ScreenMessage = StringConcatenate(ScreenMessage, "Magic Number = ",MagicNumber,NL);
   if (ManageByTradeComment) ScreenMessage = StringConcatenate(ScreenMessage, "Trade comment = ", TradeComment,NL);
   
   if (BasketClosureTP)
   {
      ScreenMessage = StringConcatenate(ScreenMessage, "Basket Closure Take Profit is enabled. ");
      if (BasketTPinDollars) ScreenMessage = StringConcatenate(ScreenMessage, "Taking profit at $", BasketDollarTP, " profit",NL);
      if (BasketTPasPercent) 
      {
         double ProfitPercentage=NormalizeDouble(AccountBalance() * (BasketTpPercentage/100),2);
         ScreenMessage = StringConcatenate(ScreenMessage, "Taking profit at ", BasketTpPercentage, "% profit = $",ProfitPercentage,NL);      
      }
   }//if (BasketClosureTP)
   
   if (!AutoCalcBasketTPasPercent && !BasketClosureTP) ScreenMessage = StringConcatenate(ScreenMessage, "Basket Closure Take Profit is disabled. ",NL);
   
   if (AutoCalcBasketTPasPercent)
   {
      double Profit=AutoPercentageBasketTp();
      ScreenMessage = StringConcatenate(ScreenMessage, "Auto-calculating bakset tp. Tp is $", Profit,NL);
   }
   
   if (BasketClosureSL)
   {
      ScreenMessage = StringConcatenate(ScreenMessage, "Basket Closure Stop Loss is enabled. ");
      if (BasketSLinDollars) ScreenMessage = StringConcatenate(ScreenMessage, "Stop loss is $", BasketDollarSL, " loss",NL);
      if (BasketSLasPercent) ScreenMessage = StringConcatenate(ScreenMessage, "Stop loss set to ", BasketSLPercentage, "% loss",NL);      
   
   }//if (BasketClosureSL)
   else ScreenMessage = StringConcatenate(ScreenMessage, "Basket Closure Stop Loss is disabled. ",NL);
      
   if (BasketTrailingStop)
   {
      ScreenMessage = StringConcatenate(ScreenMessage, "Basket Trailing Stop is enabled. Starting trail at +$", BasketTsAtProfit, " and trailing by ",BasketTrailPercent,"% of profit",NL);
      
   }//if (BasketTrailingStop)
  
   if (BasketJumpingStop)
   {
      ScreenMessage = StringConcatenate(ScreenMessage, "Basket jumping stop is enabled at $", BasketJumpingStopProfit,NL);
      if (BasketAddBEP) ScreenMessage = StringConcatenate(ScreenMessage, "Adding break even profit of $",BasketBreakEvenProfit,NL);
      if (DisableBasketJumpStopAfterBE)  ScreenMessage = StringConcatenate(ScreenMessage, "Disabling jumping stop after break even", NL);
      else  ScreenMessage = StringConcatenate(ScreenMessage, "Not disabling jumping stop after break even", NL);
   }//if (BasketJumpingStop)
     

   
   if (!BasketJumpingStop) ScreenMessage = StringConcatenate(ScreenMessage, "Basket jumping stop is disabled",NL);

   if (ShowComments) Comment(ScreenMessage);
}// end void MonitorBasketTrades()



bool ConfirmBasketClosure()
{
   // User has set BasketCloseImmediately to true, so confirm this choice.
   // Returns true if user confirms, else false
   
   int ret=MessageBox("Close all open trades monitored by this ea?","Question",MB_YESNO|MB_ICONQUESTION);
      if (ret==IDNO)
      {
         MessageBox("Remember to turn off BasketCloseImmediately.","Reminder");
         return(false);
      }
      
      return(true);   
   
}// End bool ConfirmBasketClosure()


void CloseBasketTrades()
{
   CloseBasket=false;         
   if (OrdersTotal() == 0) return;
      
   for (int cc=0; cc<OrdersTotal(); cc++)
   {
      if (!OrderSelect(cc,SELECT_BY_POS) ) return;
      if ((ManageByMagicNumber && OrderMagicNumber()==MagicNumber) || (ManageByTradeComment && OrderComment()==TradeComment))      
      {
         if (OrderType()==OP_BUY || OrderType()==OP_SELL)
         {
            bool result=OrderClose(OrderTicket(),OrderLots(), OrderClosePrice(), 500,CLR_NONE);
            if (!result) CloseBasket=true;
            else cc--;
         }//if (OrderType()==OP_BUY || OrderType()==OP_SELL)
         
         if (OrderType()!=OP_BUY && OrderType()!=OP_SELL && IncludePendingsAtClosure)
         {
            result=OrderDelete(OrderTicket() );
            if (!result) CloseBasket=true;
            else cc--;
         }//if (OrderType()!=OP_BUY && OrderType()!=OP_SELL && IncludePendingsAtClosure)
         
         
         
      }//if (OrderMagicNumber()==MagicNumber)
   }//for (int cc=0; cc<OrdersTotal(); cc++)
   
}// end void CloseBasketTrades()


void CalculateBasketPL()
{
   BasketProfit=0;
   NoOfBasketTrades=0;
   for (int cc=0; cc < OrdersTotal(); cc++)
   {
      OrderSelect(cc, SELECT_BY_POS);
      if ((ManageByMagicNumber && OrderMagicNumber()==MagicNumber) || (ManageByTradeComment && OrderComment()==TradeComment))
      {
         BasketProfit=BasketProfit+OrderProfit();
         NoOfBasketTrades++;
      }//if (ManageByMagicNumber && OrderMagicNumber()==MagicNumber || ManageByTradeComment && OrderComment()==TradeComment)      
   }//for (int cc=0; cc < OrdersTotal(); cc++)
   ScreenMessage="";
   ScreenMessage = StringConcatenate(ScreenMessage, "Basket profit = ", BasketProfit, NL);
   if (LockedProfit > -1) ScreenMessage = StringConcatenate(ScreenMessage, "Locked profits = ", LockedProfit, NL);
   Comment(ScreenMessage);
   
   
}// end void CalculateBasketP&L()



bool ShouldBasketCloseAtTP()
{
   // Returns true if the basket upl >= chosen closure point, else returns false.
   // BasketProfit is already calculated   

   // Is the profit >= selected profit point
   double ProfitPercentage=0;

   // First calculate whether the upl is >= the point at which the position is to close
   
   // Profit in dollars enabled
   if (BasketTPinDollars)
   {
      if(BasketProfit>=BasketDollarTP)
      {
         Comment("Profit target hit. Closing trade basket now");
         return(true);
      } 
   }
   
   // Profit as percentage of account balance enabled
   if (BasketTPasPercent)
   {
      ProfitPercentage=NormalizeDouble(AccountBalance() * (BasketTpPercentage/100),2);
      if (BasketProfit>= ProfitPercentage)
      {
         Comment("Profit target hit. Closing trade basket now");
         return(true);
      }
   }


   ScreenMessage="";
   ScreenMessage = StringConcatenate(ScreenMessage, "Basket profit = ", BasketProfit, NL);
   Comment(ScreenMessage);
   return(false);
   
}// end bool ShouldBasketCloseAtTP()


bool ShouldBasketCloseAtSL()
{
   // Returns true if the basket upl >= chosen closure point, else returns false.
   // BasketProfit is already calculated   

   // Is the profit >= selected profit point
   double ProfitPercentage=0;

   // First calculate whether the upl is >= the point at which the position is to close
   
   // Profit in dollars enabled
   if (BasketSLinDollars)
   {
      double bld = BasketDollarSL;
      if(BasketProfit<=bld)
      {
         Comment("Stop loss hit. Closing trade basket now");
         return(true);
      } 
   }
   
   // Profit as percentage of account balance enabled
   if (BasketSLasPercent)
   {
      ProfitPercentage=AccountBalance() * (BasketSLPercentage/100);
      bld = BasketProfit;
      bld = -bld;
      if (bld>= ProfitPercentage)
      {
         Comment("Stop loss hit. Closing trade basket now");
         return(true);
      }
   }

}// end bool BasketClosure=ShouldBasketCloseAtSL()



bool ShouldBasketCloseAtAutocalcTP()
   {
      // Calls the function that calculates the basket tp based on the percentages of balance set in the three 
      // no-of-trade bands, then calculates whether this figure has been reached. Returns 'true' if so, else returns false.
  
      // Put the percentage calculation routine into a separate function so it can be called during the
      // on-screen user feedback routines.
      double Profit=AutoPercentageBasketTp();
      
      if (BasketProfit >= Profit) return(true);
      else return(false);
   
   }//bool ShouldBasketCloseAtAutocalcTP()
   
   
   void BasketTrailingStopManipulation()
{
   
   //Has the market moved far enough to trigger the trailing stop?
   if (LockedProfit==-1)
   {
      if (BasketProfit >= BasketTsAtProfit)
      {
         LockedProfit = BasketProfit * (BasketTrailPercent/100);
         Alert("Basket trailing stop initiated at $",LockedProfit);
         return;
      }//if (BasketProfit >= BasketTsAtProfit}
   }//if (LockedProfit==-1)
   
   // Has the trail kicked in, and if so has the profit retraced to it
   if (LockedProfit > -1 && BasketProfit <= LockedProfit)
   {
      LockedProfit=-1;
      Comment("Basket trailing stop hit. Closing trades.");
      Alert("Basket trailing stop hit. Closing trades.");
      CloseBasket = false;
      CloseBasketTrades();
      return;
   }
      
   
   // Does a trailing stop need updating?
   if (BasketProfit >= BasketTsAtProfit)
   {
      double Trail = BasketProfit * (BasketTrailPercent/100);
      if (Trail > LockedProfit)
      {
         LockedProfit = Trail;
      }//if (Trail > LockedProfit)
   }//if (BasketProfit >= BasketTsAtProfi
    
   // Got this far, so give the user some feedback
   ScreenMessage="";
   ScreenMessage = StringConcatenate(ScreenMessage, "Basket profit = ", BasketProfit, NL);
   if (LockedProfit > -1) ScreenMessage = StringConcatenate(ScreenMessage, "Locked profits = ", LockedProfit, NL);
   else ScreenMessage = StringConcatenate(ScreenMessage, "No locked profits yet", NL);
   Comment(ScreenMessage);
      
}// end void BasketTrailingStopManipulation()


void BasketJumpingStopManipulation()
   {

      // Has the trail kicked in, and if so has the profit retraced to it
      if (LockedProfit > -1 && BasketProfit <= LockedProfit)
      {
         LockedProfit=-1;
         Comment("Basket jumping stop hit. Closing trades.");
         Alert("Basket jumping stop hit. Closing trades.");
         CloseBasket = false;
         CloseBasketTrades();
         return;
      }
      
      // Is jumping stop disabled after break even?
      if (LockedProfit > -1 && DisableBasketJumpStopAfterBE)
      {
         ScreenMessage="";
         ScreenMessage = StringConcatenate(ScreenMessage, "Basket profit = ", BasketProfit, NL);
         ScreenMessage = StringConcatenate(ScreenMessage, "Locked profits = ", LockedProfit, NL);
         Comment(ScreenMessage);   
      }
   
      // Set to breakeven the first time BasketProfit reaches\passes BasketJumpingStopProfit
      // Add BasketBEP later
      if (LockedProfit==-1)
      {
         if (BasketProfit >= BasketJumpingStopProfit)
         {
            LockedProfit=0;
            string JSmessage = "Basket jumping stop moved to breakeven ";
            if (BasketAddBEP) 
            {
               LockedProfit=LockedProfit + BasketBreakEvenProfit;
               JSmessage = StringConcatenate(JSmessage, "plus profit of $",LockedProfit);
            }//if (BasketAddBEP) 
            if (ShowAlerts) Alert(JSmessage);
         }//if (BasketProfit >= BasketJumpingStopProfit)         
      }//if (LockedProfit==-1)
   
      // Does a jumping stop need updating?
      if (BasketProfit >= LockedProfit + (BasketJumpingStopProfit*2)) 
      {
         LockedProfit = LockedProfit + BasketJumpingStopProfit;
         JSmessage= StringConcatenate("Basket trades jumping stop moved to $", LockedProfit);
         Alert(JSmessage);
      }
    
      // Got this far, so give the user some feedback
      ScreenMessage="";
      ScreenMessage = StringConcatenate(ScreenMessage, "Basket profit = ", BasketProfit, NL);
      if (LockedProfit > -1) ScreenMessage = StringConcatenate(ScreenMessage, "Locked profits = ", LockedProfit, NL);
      else ScreenMessage = StringConcatenate(ScreenMessage, "No locked profits yet", NL);
      Comment(ScreenMessage);
   
   
   
   }//End void BasketJumpingStopManipulation()  
   
   
   double AutoPercentageBasketTp()
   {

      // Calculates the basket tp based on the percentages of balance set in the three 
      // no-of-trade bands. Returns this figure for use in calculating whether the upl has reached the tp
      // and in screen feedback.
      
      double ProfitPercentage;
      
      if (NoOfBasketTrades < 8)
      {
         ProfitPercentage=NormalizeDouble(AccountBalance() * (FourToSevenTradesPercent/100),2);
      }
      
      if (NoOfBasketTrades > 7 && NoOfBasketTrades < 13)
      {
         ProfitPercentage=NormalizeDouble(AccountBalance() * (EightToTwelveTradesPercent/100),2);
      }
      
      if (NoOfBasketTrades > 12)
      {
         ProfitPercentage=NormalizeDouble(AccountBalance() * (ThirteenPlusTradesPercent/100),2);
      }
      
      return(ProfitPercentage);
   
   }//double AutoPercentageBasketTp()